package HTML::Robot::Scrapper::Log::Default;
use Moose;

has [qw/robot engine/] => ( is => 'rw', );

=head1 DESCRIPTION

The Log class is not implemented yet, but it will just be a log 4 perl

=cut

1;
